@@ -98,6 +98,8 @@ class Package extends BasePackage
     protected $isDefaultBranch = false;
     /** @var mixed[] */
     protected $transportOptions = [];
+    /** @var array{priority?: int, configure-options?: list<array{name: string, description?: string}>}|null */
+    protected $phpExt = null;
 
     /**
      * Creates a new in memory package.
@@ -590,6 +592,24 @@ class Package extends BasePackage
         return $this->includePaths;
     }
 
+    /**
+     * Sets the list of paths added to PHP's include path.
+     *
+     * @param array{priority?: int, configure-options?: list<array{name: string, description?: string}>}|null $phpExt List of directories.
+     */
+    public function setPhpExt(?array $phpExt): void
+    {
+        $this->phpExt = $phpExt;
+    }
+
+    /**
+     * @inheritDoc
+     */
+    public function getPhpExt(): ?array
+    {
+        return $this->phpExt;
+    }
+
     /**
      * Sets the notification URL
      */
